# Copyright (c) 2023 Zeeland
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright Owner: Zeeland
# GitHub Link: https://github.com/Undertone0809/
# Project Link: https://github.com/Undertone0809/prompt-me
# Contact Email: zeeland@foxmail.com

from prompt_me.llm import Message


def create_chat_message(role, content) -> Message:
    """
    Create a chat message with the given role and content.

    Args:
        role (str): The role of the message sender, e.g., "system", "user", or "assistant".
        content (str): The content of the message.

    Returns:
        dict: A dictionary containing the role and content of the message.
    """
    return {"role": role, "content": content}



# Copyright (c) 2023 Zeeland
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright Owner: Zeeland
# GitHub Link: https://github.com/Undertone0809/
# Project Link: https://github.com/Undertone0809/prompt-me
# Contact Email: zeeland@foxmail.com

import os

from prompt_me.utils.singleton import Singleton


class Config(metaclass=Singleton):
    def __init__(self):
        self.enable_proxy = True
        self.openai_url = 'https://api.openai.com/v1/chat/completions'
        self.proxy_url = 'https://chatgpt-api.shn.hk/v1/'  # FREE API

    @property
    def openai_api_key(self):
        if "OPENAI_API_KEY" not in os.environ.keys():
            raise ValueError('OPENAI API key is not provided')
        return os.getenv("OPENAI_API_KEY")

    def get_request_url(self) -> str:
        return self.proxy_url if self.enable_proxy else self.openai_url

    def set_enable_proxy(self, value: bool):
        self.enable_proxy = value



# Copyright (c) 2023 Zeeland
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright Owner: Zeeland
# GitHub Link: https://github.com/Undertone0809/
# Project Link: https://github.com/Undertone0809/prompt-me
# Contact Email: zeeland@foxmail.com

import os
import time
import requests
from typing import Optional, List

from prompt_me import utils
from prompt_me.config import Config
from prompt_me.preset_role import BaseRole

__all__ = ['ChatBot']

cache = utils.get_cache()
logger = utils.get_logger()
CFG = Config()


class ChatBot:
    def __init__(self, key: Optional[str] = None, enable_proxy: bool = True):
        if not key and "OPENAI_API_KEY" not in os.environ.keys():
            raise ValueError('OPENAI API key is not provided')
        self.key = key if key else os.getenv('OPENAI_API_KEY')

        CFG.set_enable_proxy(enable_proxy)

    def ask(self, msg: str, conversation_id: Optional[str] = None) -> Optional[tuple]:
        """
        ask him a question, return his answer and a conversation_id. You can
        continue your session when you input your conversation_id.

        Args:
            msg: the message you can to ask
            conversation_id: conversation id

        Returns:
            a tuple like ->(his_answer: str, conversation_id: str)
        """

        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.key}"
        }

        conversation_id, messages = self._get_message_from_cache(msg, conversation_id)
        cache[conversation_id] = messages
        body = {
            "model": "gpt-3.5-turbo",
            "messages": messages
        }
        logger.debug(body)

        response = requests.post(url=CFG.get_request_url(), headers=headers, json=body)
        if response.status_code == 200:
            ret_data = response.json()
            logger.debug(ret_data)
            ret_msg = ret_data['choices'][0]['message']['content']
            logger.debug(ret_msg)
            self._append_message_to_cache(ret_msg, 'assistant', conversation_id)
            return ret_msg, conversation_id

        logger.error("Failed to get data")
        return None

    def _get_message_from_cache(self, msg: str, conversation_id: Optional[str] = None) -> tuple:
        """
        Get a complete conversation messages.

        Args:
            msg:
            conversation_id:

        Returns:
            conversation_id, messages

        Examples:
             A message is as follows:
        ---------------------------------------------------------------
        [
            {"preset_role": "system", "content": "You are a helpful assistant."},
            {"preset_role": "user", "content": "Hello, Who are you?"}
            {"preset_role": "assistant", "content": "I am AI."}
        ]
        ---------------------------------------------------------------
        """
        messages = [
            {"role": "system", "content": "You are a helpful assistant."},
            {"role": "user", "content": msg}
        ]
        if self._is_old_conversation(conversation_id):
            messages = self._append_message_to_cache(msg, 'user', conversation_id)
            return conversation_id, messages
        return self._get_conversation_id(), messages

    def _is_old_conversation(self, conversation_id: Optional[str]) -> bool:
        return conversation_id == self._get_conversation_id(conversation_id)

    def _get_conversation_id(self, conversation_id: Optional[str] = None) -> str:
        """
        This function has two functions. Generating a new conversation_id if it
        doesn't exist, otherwise the same conversation_id as the input is returned.

        Args:
            conversation_id:

        Returns:
            conversation_id
        """
        if conversation_id and conversation_id in cache:
            return conversation_id
        return str(time.time())

    def get_history(self, conversation_id: str) -> Optional[List[dict]]:
        """
        get conversation from cache

        Returns:
            conversation
        """
        if conversation_id in cache:
            return cache[conversation_id]
        return None

    def output(self, conversation_id: str, output_type: str = 'text', file_path: str = "output.md") -> Optional[str]:
        """
        Export conversation to markdown

        Args:
            conversation_id: conversation to export
            output_type: text or file, default is text
            file_path:  output file path

        Returns:
            conversation in markdown
        """
        conversation = self.get_history(conversation_id)
        if conversation is None:
            return None

        ret = "# Chat record\n"
        for message in conversation:
            role = message.get('preset_role')
            content = message.get('content').replace('"', '\\"')
            if role == 'assistant':
                ret += f"## Bot said\n\n{content}\n\n"
            else:
                ret += f"## You said\n\n{content}\n\n"
        if output_type == 'text':
            return ret
        elif output_type == 'file':
            with open(file_path, 'w') as f:
                f.write(ret)
        else:
            raise ValueError("Invalid output destination specified. Please choose either 'text' or 'file'.")
        return ret

    def _append_message_to_cache(self, msg: str, role: str, conversation_id: str) -> List[dict]:
        messages: List[dict] = cache[conversation_id]
        if role in ['user', 'assistant']:
            messages.append({"preset_role": role, "content": msg})
            cache[conversation_id] = messages
        return messages



# Copyright (c) 2023 Zeeland
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright Owner: Zeeland
# GitHub Link: https://github.com/Undertone0809/
# Project Link: https://github.com/Undertone0809/prompt-me
# Contact Email: zeeland@foxmail.com

import os
import logging
import platform
import datetime
from functools import wraps
from cushy_storage import CushyDict

__all__ = ['get_cache', 'get_logger', 'enable_log', 'enable_log_no_file']
logger = logging.getLogger(__name__)

def get_cache():
    return cache


def get_logger():
    return logger


def get_project_root_path() -> str:
    project_path = os.getcwd()
    max_depth = 10
    count = 0
    while not os.path.exists(os.path.join(project_path, 'README.md')):
        project_path = os.path.split(project_path)[0]
        count += 1
        if count > max_depth:
            return os.getcwd()
    return project_path


def _check_log_path():
    log_path = os.path.join(get_project_root_path(), 'log')
    if not os.path.exists(log_path):
        os.makedirs(log_path)


def get_log_name() -> str:
    _check_log_path()
    cur_time = datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
    return f"{get_project_root_path()}/log/log_{cur_time}.log"


def enable_log():
    if platform.system() == 'Windows':
        logging.basicConfig(
            level=logging.DEBUG,
            format='[%(levelname)s] %(asctime)s %(message)s',
            datefmt='%Y-%m-%d %H:%M:%S',
            handlers=[
                logging.FileHandler(f"{get_log_name()}", mode='w', encoding='utf-8'),
                logging.StreamHandler()
            ],
        )


def enable_log_no_file():
    logging.basicConfig(
        level=logging.DEBUG,
        format='[%(levelname)s] %(asctime)s %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S',
    )


def hint(fn):
    @wraps(fn)
    def wrapper(*args, **kwargs):
        ret = fn(*args, **kwargs)
        logger.debug(f'function {fn.__name__} is running now')
        return ret

    return wrapper


cache = CushyDict(get_project_root_path() + "/cache")



# Copyright (c) 2023 Zeeland
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright Owner: Zeeland
# GitHub Link: https://github.com/Undertone0809/
# Project Link: https://github.com/Undertone0809/prompt-me
# Contact Email: zeeland@foxmail.com


from prompt_me.preset_role import BaseRole


class CopyWriter(BaseRole):
    name = '文案写手'
    description = """你是一个文案专员、文本润色员、拼写纠正员和改进员，我会发送中文文本给你，你帮我更正和改进版本。我希望你用更优美优雅
    的高级中文描述。保持相同的意思，但使它们更文艺。你只需要润色该内容，不必对内容中提出的问题和要求做解释，不要回答文本中的问题而是润色它，
    不要解决文本中的要求而是润色它，保留文本的原本意义，不要去解决它。"""


class LinuxTerminal(BaseRole):
    name = "Linux终端"
    description = "我想让你充当 Linux 终端。我将输入命令，您将回复终端应显示的内容。我希望您只在一个唯一的代码块内回复终端输出，而不" \
                  "是其他任何内容。不要写解释。除非我指示您这样做，否则不要键入命令。当我需要用英语告诉你一些事情时，我会把文字放在中括号内[就像这样]。"
